'''`bestdori.ayachan.utils.utils`

ayachan 工具库模块'''


API = {
    'map-info': 'v2/map-info/',
    'bestdori': 'v2/map-info/bestdori/{id}',
    'DiffAnalysis': 'DiffAnalysis',
    'levels': 'https://sonolus.ayachan.fun/test/sonolus/levels'
}
'''ayachan API 集合'''
